@@ -44,7 +44,7 @@
 #include <iomanip>
 #include <istream>
 #include <limits>
-#include <map>
+#include <memory>
 #include <sstream>
 #include <stdexcept>
 #include <string>
@@ -54,6 +54,7 @@
 
 #include <iostream>
 
+#include "NullOStream.hpp"
 #include "pdal_util_export.hpp"
 
 namespace pdal
@@ -64,8 +65,8 @@ namespace Utils
     /**
      * \brief Clamp value to given bounds.
      *
-     * Clamps the input value t to bounds specified by min and max. Used to ensure
-     * that row and column indices remain within valid bounds.
+     * Clamps the input value t to bounds specified by min and max.
+     * Used to ensure that row and column indices remain within valid bounds.
      *
      * \param t the input value.
      * \param min the lower bound.
@@ -73,9 +74,9 @@ namespace Utils
      * \return the value to clamped to the given bounds.
      */
     template <class T>
-    PDAL_DLL const T& clamp(const T& t, const T& min, const T& max)
+    PDAL_DLL const T& clamp(const T& t, const T& minimum, const T& maximum)
     {
-        return ((t < min) ? min : ((t > max) ? max : t));
+        return ((t < minimum) ? minimum : ((t > maximum) ? maximum : t));
     }
 
     /**
@@ -431,19 +432,12 @@ namespace Utils
     */
     PDAL_DLL std::string hexDump(const char *buf, size_t count);
 
-    /**
-      Generate a backtrace as a list of strings.
-
-      \return  List of functions at the point of the call.
-    */
-    PDAL_DLL std::vector<std::string> backtrace();
-
     /**
       Count the number of characters in a string that meet a predicate.
 
       \param s  String in which to start counting characters.
       \param p  Position in input string at which to start counting.
-      \param pred  Unary predicte that tests a character.
+      \param pred  Unary predicate that tests a character.
       \return  Then number of characters matching the predicate.
     */
     template<typename PREDICATE>
@@ -451,11 +445,22 @@ namespace Utils
     extract(const std::string& s, std::string::size_type p, PREDICATE pred)
     {
         std::string::size_type count = 0;
-        while (pred(s[p++]))
+        while (p < s.size() && pred(s[p++]))
             count++;
         return count;
     }
 
+    /**
+      Count the number of characters spaces in a string at a position.
+
+      \param s  String in which to start counting characters.
+      \param p  Position in input string at which to start counting.
+      \return  Then number of space-y characters matching the predicate.
+    */
+    PDAL_DLL inline std::string::size_type
+    extractSpaces(const std::string& s, std::string::size_type p)
+        { return extract(s, p, (int(*)(int))std::isspace); }
+
     /**
       Split a string into substrings based on a predicate.  Characters
       matching the predicate are discarded.
@@ -554,6 +559,20 @@ namespace Utils
         return split2(s, pred);
     }
 
+    /**
+      Perform shell-style word expansion (break a string into arguments).
+      This only does simple handling of quoted values and backslashes
+      and doesn't support fancier shell behavior.  Use the real wordexp()
+      if you need all that.  The behavior of escaped values in a string
+      was surprising to me, so try the shell first if you think you've
+      found a problem.
+
+      \param s  Input string to parse.
+      \return  List of arguments.
+    */
+    PDAL_DLL std::vector<std::string>
+    simpleWordexp(const std::string& s);
+
     /**
       Return a string representation of a type specified by the template
       argument.
@@ -566,16 +585,16 @@ namespace Utils
 
     struct RedirectStream
     {
-        RedirectStream() : m_out(NULL), m_out2(NULL), m_buf(NULL)
+        RedirectStream() : m_out(NULL), m_buf(NULL), m_null(new NullOStream)
         {}
 
         std::ofstream *m_out;
-        std::ostream *m_out2;
         std::streambuf *m_buf;
+        std::unique_ptr<NullOStream> m_null;
     };
 
     /**
-      Redirect a stream to some other stream.
+      Redirect a stream to some other stream, by default a null stream.
 
       \param out   Stream to redirect.
       \param dst   Destination stream.
@@ -585,21 +604,34 @@ namespace Utils
     {
         RedirectStream redir;
 
-        redir.m_out2 = &dst;
         redir.m_buf = out.rdbuf();
-        out.rdbuf(redir.m_out2->rdbuf());
+        out.rdbuf(dst.rdbuf());
         return redir;
     }
 
     /**
-      Redirect a stream to some file, by default /dev/null.
+      Redirect a stream to a null stream.
+
+      \param out   Stream to redirect.
+      \return  Context for stream restoration (see \ref restore()).
+    */
+    inline RedirectStream redirect(std::ostream& out)
+    {
+        RedirectStream redir;
+
+        redir.m_buf = out.rdbuf();
+        out.rdbuf(redir.m_null->rdbuf());
+        return redir;
+    }
+
+    /**
+      Redirect a stream to some file.
 
       \param out   Stream to redirect.
       \param file  Name of file where stream should be redirected.
       \return  Context for stream restoration (see \ref restore()).
     */
-    inline RedirectStream redirect(std::ostream& out,
-        const std::string& file = "/dev/null")
+    inline RedirectStream redirect(std::ostream& out, const std::string& file)
     {
         RedirectStream redir;
 
@@ -616,13 +648,12 @@ namespace Utils
       \param redir RedirectStream returned from corresponding
         \ref redirect() call.
     */
-    inline void restore(std::ostream& out, RedirectStream redir)
+    inline void restore(std::ostream& out, RedirectStream& redir)
     {
         out.rdbuf(redir.m_buf);
         if (redir.m_out)
             redir.m_out->close();
         redir.m_out = NULL;
-        redir.m_out2 = NULL;
         redir.m_buf = NULL;
     }
 
@@ -647,8 +678,8 @@ namespace Utils
         }
 
         return std::is_same<double, T_OUT>::value ||
-           (in >= static_cast<double>(std::numeric_limits<T_OUT>::lowest()) &&
-            in <= static_cast<double>(std::numeric_limits<T_OUT>::max()));
+            (in >= static_cast<double>(std::numeric_limits<T_OUT>::lowest()) &&
+             in <= static_cast<double>((std::numeric_limits<T_OUT>::max)()));
     }
 
     /**
@@ -688,7 +719,7 @@ namespace Utils
         if (std::is_integral<T_OUT>::value)
             in = static_cast<T_IN>(sround((double)in));
         if ((std::is_same<T_OUT, double>::value) ||
-            (in <= static_cast<double>(std::numeric_limits<T_OUT>::max()) &&
+            (in <= static_cast<double>((std::numeric_limits<T_OUT>::max)()) &&
              in >= static_cast<double>(std::numeric_limits<T_OUT>::lowest())))
         {
             out = static_cast<T_OUT>(in);
@@ -697,6 +728,28 @@ namespace Utils
         return false;
     }
 
+    /**
+      Convert a numeric value from double to float.  Specialization to handle
+      NaN.
+
+      \param in  Value to convert.
+      \param out  Converted value.
+      \return  \c true if the conversion was successful, \c false if the
+        datatypes/input value don't allow conversion.
+    */
+    template<>
+    inline bool numericCast(double in, float& out)
+    {
+        if ((in <= static_cast<double>((std::numeric_limits<float>::max)()) &&
+            in >= static_cast<double>(std::numeric_limits<float>::lowest())) ||
+            std::isnan(in))
+        {
+            out = static_cast<float>(in);
+            return true;
+        }
+        return false;
+    }
+
     /**
       Convert a value to its string representation by writing to a stringstream.
 
@@ -725,10 +778,16 @@ namespace Utils
       \param from  Value to convert.
       \return  String representation of numeric value.
     */
-    inline std::string toString(double from)
+    inline std::string toString(double from, size_t precision = 10)
     {
         std::ostringstream oss;
-        oss << std::setprecision(10) << from;
+        // Standardize nan/inf output to the JAVA property names because
+        // when we convert to a string, we usually convert to JSON.
+        if (std::isnan(from))
+            return "NaN";
+        if (std::isinf(from))
+            return (from < 0 ? "-Infinity" : "Infinity");
+        oss << std::setprecision(precision) << from;
         return oss.str();
     }
 
@@ -835,6 +894,23 @@ namespace Utils
     inline std::string toString(signed char from)
         { return std::to_string((int)from); }
 
+
+    template<typename T>
+    bool fromString(const std::string& from, T* & to)
+    {
+        void *v;
+        // Uses sscanf instead of operator>>(istream, void*&) as a workaround
+        // for https://bugs.llvm.org/show_bug.cgi?id=19740, which presents with
+        // clang-800.0.42.1 for x86_64-apple-darwin15.6.0.
+        int result = sscanf(from.c_str(), "%p", &v);
+        if (result != 1) {
+            return false;
+        }
+        to = reinterpret_cast<T*>(v);
+        return true;
+    }
+
+
     /**
       Convert a string to a value by reading from a string stream.
 
@@ -873,13 +949,13 @@ namespace Utils
         {
             int i = std::stoi(s);
             if (i >= std::numeric_limits<char>::lowest() &&
-                    i <= std::numeric_limits<char>::max())
+                    i <= (std::numeric_limits<char>::max)())
             {
                 to = static_cast<char>(i);
                 return true;
             }
         }
-        catch (std::invalid_argument) // Character that isn't a number?
+        catch (std::invalid_argument&) // Character that isn't a number?
         {
             if (s.length() == 1)
             {
@@ -905,13 +981,13 @@ namespace Utils
         {
             int i  = std::stoi(s);
             if (i >= std::numeric_limits<unsigned char>::lowest() &&
-                i <= std::numeric_limits<unsigned char>::max())
+                i <= (std::numeric_limits<unsigned char>::max)())
             {
                 to = static_cast<unsigned char>(i);
                 return true;
             }
         }
-        catch (std::invalid_argument) // Character that isn't a number?
+        catch (std::invalid_argument&) // Character that isn't a number?
         {
             if (s.length() == 1)
             {
@@ -937,13 +1013,13 @@ namespace Utils
         {
             int i = std::stoi(s);
             if (i >= std::numeric_limits<signed char>::lowest() &&
-                    i <= std::numeric_limits<signed char>::max())
+                i <= (std::numeric_limits<signed char>::max)())
             {
                 to = static_cast<signed char>(i);
                 return true;
             }
         }
-        catch (std::invalid_argument) // Character that isn't a number?
+        catch (std::invalid_argument&) // Character that isn't a number?
         {
             if (s.length() == 1)
             {
